@@ -52,7 +52,7 @@ import java.util.*;
  * @author Bing Ran
  * @author <a href="mailto:scottstirling@rcn.com">Scott Stirling</a>
  * @author <a href="mailto:blackdrag@gmx.org">Jochen Theodorou</a>
- * @version $Revision: 9762 $
+ * @version $Revision: 10686 $
  */
 public class GroovyClassLoader extends URLClassLoader {
 
@@ -196,7 +196,13 @@ public class GroovyClassLoader extends URLClassLoader {
      * @return the main class defined in the given script
      */
     public Class parseClass(String text, String fileName) throws CompilationFailedException {
-        return parseClass(new ByteArrayInputStream(text.getBytes()), fileName);
+        byte[] bytes = null;
+        try {
+            bytes = text.getBytes(config.getSourceEncoding());
+        } catch (UnsupportedEncodingException e) {
+            throw new CompilationFailedException(1,null,e);
+        }
+        return parseClass(new ByteArrayInputStream(bytes), fileName);
     }
 
     /**
@@ -206,7 +212,7 @@ public class GroovyClassLoader extends URLClassLoader {
      * @return the main class defined in the given script
      */
     public Class parseClass(String text) throws CompilationFailedException {
-        return parseClass(new ByteArrayInputStream(text.getBytes()), "script" + System.currentTimeMillis() + ".groovy");
+        return parseClass(text, "script" + System.currentTimeMillis() + ".groovy");
     }
 
     /**
