@@ -386,7 +386,7 @@ public class TemplateServlet extends AbstractHttpServlet {
         // Prepare the response buffer content type _before_ getting the writer.
         // and set status code to ok
         //
-        response.setContentType(CONTENT_TYPE_TEXT_HTML);
+        response.setContentType(CONTENT_TYPE_TEXT_HTML+"; charset="+encoding);
         response.setStatus(HttpServletResponse.SC_OK);
 
         //
