@@ -25,6 +25,8 @@ import org.codehaus.groovy.ast.stmt.Statement;
 import org.codehaus.groovy.classgen.Verifier;
 import org.codehaus.groovy.control.messages.ExceptionMessage;
 import org.codehaus.groovy.syntax.Types;
+import org.codehaus.groovy.GroovyBugError;
+import org.objectweb.asm.Opcodes;
 
 import java.io.File;
 import java.io.IOException;
@@ -62,6 +64,74 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
 
     private Map genericParameterNames = new HashMap();
 
+    /**
+     * we use ConstructedClassWithPackage to limit the resolving the compiler
+     * does when combining package names and class names. The idea
+     * that if we use a package, then we do not want to replace the
+     * '.' with a '$' for the package part, only for the class name
+     * part. There is also the case of a imported class, so this logic
+     * can't be done in these cases...
+     */
+    private static class ConstructedClassWithPackage extends ClassNode {
+        String prefix;
+        String className;
+        public ConstructedClassWithPackage(String pkg, String name) {
+            super(pkg+name, Opcodes.ACC_PUBLIC,ClassHelper.OBJECT_TYPE);
+            isPrimaryNode = false;
+            this.prefix = pkg;
+            this.className = name;
+        }
+        public String getName() {
+            if (redirect()!=this) return super.getName();
+            return prefix+className;
+        }
+        public boolean hasPackageName() {
+            if (redirect()!=this) return super.hasPackageName();
+            return className.indexOf('.')!=-1;
+        }
+        public String setName(String name) {
+            if (redirect()!=this) {
+                return super.setName(name);
+            } else {
+                throw new GroovyBugError("ConstructedClassWithPackage#setName should not be called");
+            }
+        }
+    }
+
+     /**
+     * we use LowerCaseClass to limit the resolving the compiler
+     * does for vanilla names starting with a lower case letter. The idea
+     * that if we use a vanilla name with a lower case letter, that this
+     * is in most cases no class. If it is a class the class needs to be
+     * imported explicitly. The efffect is that in an expression like
+     * "def foo = bar" we do not have to use a loadClass call to check the
+     * name foo and bar for being classes. Instead we will ask the module
+     * for an alias for this name which is much faster.
+     */
+    private static class LowerCaseClass extends ClassNode {
+        String className;
+        public LowerCaseClass(String name) {
+            super(name, Opcodes.ACC_PUBLIC,ClassHelper.OBJECT_TYPE);
+            isPrimaryNode = false;
+            this.className = name;
+        }
+        public String getName() {
+            if (redirect()!=this) return super.getName();
+            return className;
+        }
+        public boolean hasPackageName() {
+            if (redirect()!=this) return super.hasPackageName();
+            return false;
+        }
+        public String setName(String name) {
+            if (redirect()!=this) {
+                return super.setName(name);
+            } else {
+                throw new GroovyBugError("ConstructedClassWithPackage#setName should not be called");
+            }
+        }
+    }
+
     public ResolveVisitor(CompilationUnit cu) {
         compilationUnit = cu;
     }
@@ -112,15 +182,18 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     }
 
     private boolean resolveToInner (ClassNode type) {
-        String name = type.getName(), saved = name;
+        // we do not do our name mangling to find an inner class
+        // if the type is a ConstructedClassWithPackage, because in this case we
+        // are resolving the name at a different place already
+        if (type instanceof ConstructedClassWithPackage) return false;
+        String name = type.getName();
+        String saved = name;
         while (true) {
             int len = name.lastIndexOf('.');
-            if (len == -1)
-              break;
+            if (len == -1) break;
             name = name.substring(0,len) + "$" + name.substring(len+1);
             type.setName(name);
-            if (resolve(type))
-              return true;
+            if (resolve(type)) return true;
         }
         type.setName(saved);
         return false;
@@ -175,7 +248,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             return true;
         }
 
-        return resolveFromModule(type, testModuleImports) ||
+        return  resolveFromModule(type, testModuleImports) ||
                 resolveFromCompileUnit(type) ||
                 resolveFromDefaultImports(type, testDefaultImports) ||
                 resolveFromStaticInnerClasses(type, testStaticInnerClasses) ||
@@ -228,6 +301,16 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
 
     private boolean resolveToScript(ClassNode type) {
         String name = type.getName();
+
+        // We do not need to check instances of LowerCaseClass
+        // to be a script, because unless there was an import for
+        // for this we  do not lookup these cases. This was a decision
+        // made on the mailing list. To ensure we will not visit this
+        // method again we set a NO_CLASS for this name
+        if (type instanceof LowerCaseClass) {
+            cachedClasses.put(name, NO_CLASS);
+        }
+        
         if (cachedClasses.get(name) == NO_CLASS) return false;
         if (cachedClasses.get(name) == SCRIPT) cachedClasses.put(name, NO_CLASS);
         if (name.startsWith("java.")) return type.isResolved();
@@ -248,6 +331,7 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                 Class cls = type.getTypeClass();
                 // if the file is not newer we don't want to recompile
                 if (!isSourceNewer(url, cls)) return true;
+                // since we came to this, we want to recompile
                 cachedClasses.remove(type.getName());
                 type.setRedirect(null);
             }
@@ -259,22 +343,44 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         return type.isResolved();
     }
 
+    private String replaceLastPoint(String name) {
+        int lastPoint = name.lastIndexOf('.');
+        name = new StringBuffer()
+                .append(name.substring(0, lastPoint))
+                .append("$")
+                .append(name.substring(lastPoint + 1))
+                .toString();
+        return name;
+    }
 
     private boolean resolveFromStaticInnerClasses(ClassNode type, boolean testStaticInnerClasses) {
+        // a class consisting of a vanilla name can never be
+        // a static inner class, because at last one dot is
+        // required for this. Example: foo.bar -> foo$bar
+        if (type instanceof LowerCaseClass) return false;
+
         // try to resolve a public static inner class' name
         testStaticInnerClasses &= type.hasPackageName();
         if (testStaticInnerClasses) {
-            String name = type.getName();
-            String replacedPointType = name;
-            int lastPoint = replacedPointType.lastIndexOf('.');
-            replacedPointType = new StringBuffer()
-                    .append(replacedPointType.substring(0, lastPoint))
-                    .append("$")
-                    .append(replacedPointType.substring(lastPoint + 1))
-                    .toString();
-            type.setName(replacedPointType);
-            if (resolve(type, false, true, true)) return true;
-            type.setName(name);
+            if (type instanceof ConstructedClassWithPackage) {
+                // we replace '.' only in the className part
+                // with '$' to find an inner class. The case that
+                // the packageis really a class is handled else where
+                ConstructedClassWithPackage tmp = (ConstructedClassWithPackage) type;
+                String name = ((ConstructedClassWithPackage) type).className;
+                tmp.className = replaceLastPoint(name);
+                if (resolve(tmp, false, true, true)) {
+                    type.setRedirect(tmp.redirect());
+                    return true;
+                }
+                tmp.className = name;
+            }   else {
+                String name = type.getName();
+                String replacedPointType = replaceLastPoint(name);
+                type.setName(replacedPointType);
+                if (resolve(type, false, true, true)) return true;
+                type.setName(name);
+            }
         }
         return false;
     }
@@ -282,14 +388,25 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     private boolean resolveFromDefaultImports(ClassNode type, boolean testDefaultImports) {
         // test default imports
         testDefaultImports &= !type.hasPackageName();
+        // we do not resolve a vanilla name starting with a lower case letter
+        // try to resolve against adefault import, because we know that the
+        // default packages do not contain classes like these
+        testDefaultImports &= !(type instanceof LowerCaseClass);
         if (testDefaultImports) {
             for (int i = 0, size = DEFAULT_IMPORTS.length; i < size; i++) {
                 String packagePrefix = DEFAULT_IMPORTS[i];
                 String name = type.getName();
-                String fqn = packagePrefix + name;
-                type.setName(fqn);
-                if (resolve(type, false, false, false)) return true;
-                type.setName(name);
+                // We limit the inner class lookups here by using ConstructedClassWithPackage.
+                // This way only the name will change, the packagePrefix will
+                // not be included in the lookup. The case where the
+                // packagePrefix is really a class is handled else where.
+                // WARNING: This code does not expect a class that has an static
+                //          inner class in DEFAULT_IMPORTS
+                ConstructedClassWithPackage tmp =  new ConstructedClassWithPackage(packagePrefix,name);
+                if (resolve(tmp, false, false, false)) {
+                    type.setRedirect(tmp.redirect());
+                    return true;
+                }
             }
             String name = type.getName();
             if (name.equals("BigInteger")) {
@@ -320,8 +437,8 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         n.setRedirect(cn);
     }
 
-    private void ambiguousClass(ClassNode type, ClassNode iType, String name, boolean resolved) {
-        if (resolved && !type.getName().equals(iType.getName())) {
+    private void ambiguousClass(ClassNode type, ClassNode iType, String name) {
+        if (type.getName().equals(iType.getName())) {
             addError("reference to " + name + " is ambiguous, both class " + type.getName() + " and " + iType.getName() + " match", type);
         } else {
             type.setRedirect(iType);
@@ -329,6 +446,11 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     }
 
     private boolean resolveAliasFromModule(ClassNode type) {
+        // In case of getting a ConstructedClassWithPackage here we do not do checks for partial
+        // matches with imported classes. The ConstructedClassWithPackage is already a constructed
+        // node and any subclass resolving will then take elsewhere place
+        if (type instanceof ConstructedClassWithPackage) return false;
+
         ModuleNode module = currentClass.getModule();
         if (module == null) return false;
         String name = type.getName();
@@ -346,18 +468,31 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         while (true) {
             pname = name.substring(0, index);
             ClassNode aliasedNode = module.getImport(pname);
+
             if (aliasedNode != null) {
                 if (pname.length() == name.length()) {
-                    // full match, no need to create a new class
+                    // full match
+
+                    // We can compare here by length, because pname is always
+                    // a sbustring of name, so same length means they are equal.
                     type.setRedirect(aliasedNode);
                     return true;
                 } else {
                     //partial match
-                    String newName = aliasedNode.getName() + name.substring(pname.length());
-                    type.setName(newName);
-                    if (resolve(type, true, true, true)) return true;
-                    // was not resolved so it was a fake match
-                    type.setName(name);
+
+                    // At this point we know that we have a match for pname. This may
+                    // mean, that name[pname.length()..<-1] is a static inner class.
+                    // For this the rest of the name does not need any dots in its name.
+                    // It is either completely a inner static class or it is not.
+                    // Since we do not want to have useless lookups we create the name
+                    // completely and use a ConstructedClassWithPackage to prevent lookups against the package.
+                    String className = aliasedNode.getNameWithoutPackage() + '$' +
+                                       name.substring(pname.length()+1).replace('.', '$');
+                    ConstructedClassWithPackage tmp = new ConstructedClassWithPackage(aliasedNode.getPackageName()+".", className);
+                    if (resolve(tmp, true, true, false)) {
+                        type.setRedirect(tmp.redirect());
+                        return true;
+                    }
                 }
             }
             index = pname.lastIndexOf('.');
@@ -367,12 +502,30 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
     }
 
     private boolean resolveFromModule(ClassNode type, boolean testModuleImports) {
+        // we decided if we have a vanilla name starting with a lower case
+        // letter that we will not try to resolve this name against .*
+        // imports. Instead a full import is needed for these.
+        // resolveAliasFromModule will do this check for us. This method
+        // does also check the module contains a class in the same package
+        // of this name. This check is not done for vanilla names starting
+        // with a lower case letter anymore
+        if (type instanceof LowerCaseClass) {
+            return resolveAliasFromModule(type);
+        }
+
         String name = type.getName();
         ModuleNode module = currentClass.getModule();
         if (module == null) return false;
 
-        if (!type.hasPackageName() && module.hasPackageName()) {
+        boolean newNameUsed = false;
+        // we add a package if there is none yet and the module has one. But we
+        // do not add that if the type is a ConstructedClassWithPackage. The code in ConstructedClassWithPackage
+        // hasPackageName() will return true if ConstructedClassWithPackage#className has no dots.
+        // but since the prefix may have them and the code there does ignore that
+        // fact. We check here for ConstructedClassWithPackage.
+        if (!type.hasPackageName() && module.hasPackageName() && !(type instanceof ConstructedClassWithPackage)) {
             type.setName(module.getPackageName() + name);
+            newNameUsed = true;
         }
         // look into the module node if there is a class with that name
         List moduleClasses = module.getClasses();
@@ -383,39 +536,66 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                 return true;
             }
         }
-        type.setName(name);
+        if (newNameUsed) type.setName(name);
 
         if (testModuleImports) {
             if (resolveAliasFromModule(type)) return true;
 
-            boolean resolved = false;
             if (module.hasPackageName()) {
-                // check package this class is defined in
-                type.setName(module.getPackageName() + name);
-                resolved = resolve(type, false, false, false);
+                // check package this class is defined in. The usage of ConstructedClassWithPackage here
+                // means, that the module package will not be involved when the
+                // compiler tries to find an inner class.
+                ConstructedClassWithPackage tmp =  new ConstructedClassWithPackage(module.getPackageName(),name);
+                if (resolve(tmp, false, false, false)) {
+                    type.setRedirect(tmp.redirect());
+                    return true;
+                }
             }
+
             // check module node imports packages
             List packages = module.getImportPackages();
-            ClassNode iType = ClassHelper.makeWithoutCaching(name);
             for (Iterator iter = packages.iterator(); iter.hasNext();) {
                 String packagePrefix = (String) iter.next();
-                String fqn = packagePrefix + name;
-                iType.setName(fqn);
-                if (resolve(iType, false, false, true)) {
-                    ambiguousClass(type, iType, name, resolved);
+                // We limit the inner class lookups here by using ConstructedClassWithPackage.
+                // This way only the name will change, the packagePrefix will
+                // not be included in the lookup. The case where the
+                // packagePrefix is really a class is handled else where.
+                ConstructedClassWithPackage tmp =  new ConstructedClassWithPackage(packagePrefix,name);
+                if (resolve(tmp, false, false, true)) {
+                    ambiguousClass(type, tmp, name);
+                    type.setRedirect(tmp.redirect());
                     return true;
                 }
-                iType.setName(name);
             }
-            if (!resolved) type.setName(name);
-            return resolved;
         }
         return false;
     }
 
     private boolean resolveToClass(ClassNode type) {
         String name = type.getName();
-        if (cachedClasses.get(name) == NO_CLASS) return false;
+
+        // We do not need to check instances of LowerCaseClass
+        // to be a Class, because unless there was an import for
+        // for this we  do not lookup these cases. This was a decision
+        // made on the mailing list. To ensure we will not visit this
+        // method again we set a NO_CLASS for this name
+        if (type instanceof LowerCaseClass) {
+            cachedClasses.put(name,NO_CLASS);
+        }
+
+        // We use here the class cahce cachedClasses to prevent
+        // calls to ClassLoader#loadClass. disabling this cache will
+        // cause a major performance hit. Unlike at the end of this
+        // method we do not return true or false depending on if we
+        // want to recompile or not. If the class was cached, then
+        // we do not want to recompile, recompilation is already
+        // scheduled then
+        Object cached = cachedClasses.get(name);
+        if (cached == NO_CLASS) return false;
+        // cached == SCRIPT should not happen here!
+        if (cached == SCRIPT) throw new GroovyBugError("name "+name+" was marked as script, but was not resolved as such");
+        if (cached != null) return true;
+
         if (currentClass.getModule().hasPackageName() && name.indexOf('.') == -1) return false;
         GroovyClassLoader loader = compilationUnit.getClassLoader();
         Class cls;
@@ -442,40 +622,46 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         if (cls == null) return false;
         cachedClasses.put(name, cls);
         setClass(type, cls);
-        //NOTE: we return false here even if we found a class,
-        //but we want to give a possible script a chance to recompile.
-        //this can only be done if the loader was not the instance
-        //defining the class.
+        //NOTE: we might return false here even if we found a class,
+        //      because  we want to give a possible script a chance to
+        //      recompile. This can only be done if the loader was not
+        //      the instance defining the class.
         return cls.getClassLoader() == loader;
     }
 
 
     public Expression transform(Expression exp) {
         if (exp == null) return null;
+        Expression ret = null;
         if (exp instanceof VariableExpression) {
-            return transformVariableExpression((VariableExpression) exp);
+            ret = transformVariableExpression((VariableExpression) exp);
         } else if (exp.getClass() == PropertyExpression.class) {
-            return transformPropertyExpression((PropertyExpression) exp);
+            ret = transformPropertyExpression((PropertyExpression) exp);
         } else if (exp instanceof DeclarationExpression) {
-            return transformDeclarationExpression((DeclarationExpression) exp);
+            ret = transformDeclarationExpression((DeclarationExpression) exp);
         } else if (exp instanceof BinaryExpression) {
-            return transformBinaryExpression((BinaryExpression) exp);
+            ret = transformBinaryExpression((BinaryExpression) exp);
         } else if (exp instanceof MethodCallExpression) {
-            return transformMethodCallExpression((MethodCallExpression) exp);
+            ret = transformMethodCallExpression((MethodCallExpression) exp);
         } else if (exp instanceof ClosureExpression) {
-            return transformClosureExpression((ClosureExpression) exp);
+            ret = transformClosureExpression((ClosureExpression) exp);
         } else if (exp instanceof ConstructorCallExpression) {
-            return transformConstructorCallExpression((ConstructorCallExpression) exp);
+            ret = transformConstructorCallExpression((ConstructorCallExpression) exp);
         } else if (exp instanceof AnnotationConstantExpression) {
-            return transformAnnotationConstantExpression((AnnotationConstantExpression) exp);
+            ret = transformAnnotationConstantExpression((AnnotationConstantExpression) exp);
         } else {
             resolveOrFail(exp.getType(), exp);
-            return exp.transformExpression(this);
+            ret = exp.transformExpression(this);
         }
+        if (ret!=null && ret!=exp) ret.setSourcePosition(exp);
+        return ret;
     }
 
     private String lookupClassName(PropertyExpression pe) {
+        boolean doInitialClassTest=true;
         String name = "";
+        // this loop builds a name from right to left each name part
+        // separated by "."
         for (Expression it = pe; it != null; it = ((PropertyExpression) it).getObjectExpression()) {
             if (it instanceof VariableExpression) {
                 VariableExpression ve = (VariableExpression) it;
@@ -483,7 +669,20 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                 if (ve == VariableExpression.SUPER_EXPRESSION || ve == VariableExpression.THIS_EXPRESSION) {
                     return null;
                 }
-                name = ve.getName() + "." + name;
+                String varName = ve.getName();
+                if (doInitialClassTest) {
+                    // we are at the first name part. This is the right most part.
+                    // If this part is in lower case, then we do not need a class
+                    // check. other parts of the property expression will be tested
+                    // by a different method call to this method, so foo.Bar.bar
+                    // can still be resolved to the class foo.Bar and the static
+                    // field bar.
+                    if (!testVanillaNameForClass(varName)) return null;
+                    doInitialClassTest= false;
+                    name = varName;
+                } else {
+                    name = varName + "." + name;
+                }
                 break;
             }
             // anything other than PropertyExpressions, ClassExpression or
@@ -497,11 +696,23 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
                 if (propertyPart == null || propertyPart.equals("class")) {
                     return null;
                 }
-                name = propertyPart + "." + name;
+                if (doInitialClassTest) {
+                    // we are at the first name part. This is the right most part.
+                    // If this part is in lower case, then we do not need a class
+                    // check. other parts of the property expression will be tested
+                    // by a different method call to this method, so foo.Bar.bar
+                    // can still be resolved to the class foo.Bar and the static
+                    // field bar.
+                    if (!testVanillaNameForClass(propertyPart)) return null;
+                    doInitialClassTest= false;
+                    name = propertyPart;
+                } else {
+                    name = propertyPart + "." + name;
+                }
             }
         }
-        if (name.length() > 0) return name.substring(0, name.length() - 1);
-        return null;
+        if (name.length() == 0) return null;
+        return name;
     }
 
     // iterate from the inner most to the outer and check for classes
@@ -553,19 +764,29 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         inPropertyExpression = ipe;
 
         boolean spreadSafe = pe.isSpreadSafe();
+        PropertyExpression old = pe;
         pe = new PropertyExpression(objectExpression, property, pe.isSafe());
         pe.setSpreadSafe(spreadSafe);
+        pe.setSourcePosition(old);
 
         String className = lookupClassName(pe);
         if (className != null) {
             ClassNode type = ClassHelper.make(className);
-            if (resolve(type)) return new ClassExpression(type);
+            if (resolve(type)) {
+                Expression ret =  new ClassExpression(type);
+                ret.setSourcePosition(pe);
+                return ret;
+            }
         }
         if (objectExpression instanceof ClassExpression && pe.getPropertyAsString() != null) {
             // possibly an inner class
             ClassExpression ce = (ClassExpression) objectExpression;
             ClassNode type = ClassHelper.make(ce.getType().getName() + "$" + pe.getPropertyAsString());
-            if (resolve(type, false, false, false)) return new ClassExpression(type);
+            if (resolve(type, false, false, false)) {
+                Expression ret = new ClassExpression(type);
+                ret.setSourcePosition(ce);
+                return ret;
+            }
         }
         Expression ret = pe;
         if (isTopLevelProperty) ret = correctClassClassChain(pe);
@@ -576,9 +797,24 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         if (ve.getName().equals("this")) return VariableExpression.THIS_EXPRESSION;
         if (ve.getName().equals("super")) return VariableExpression.SUPER_EXPRESSION;
         Variable v = ve.getAccessedVariable();
-        if (v instanceof DynamicVariable) {
-            ClassNode t = ClassHelper.make(ve.getName());
-            if (resolve(t)) {
+        if (v instanceof DynamicVariable){
+            String name = ve.getName();
+            ClassNode t = ClassHelper.make(name);
+            // asking isResolved here allows to check if a primitive
+            // type name like "int" was used to make t. In such a case
+            // we have nothing left to do.
+            boolean isClass = t.isResolved();
+            if (!isClass) {
+                // It was no primitive type, so next we see if the name,
+                // which is a vanilla name, starts with a lower case letter.
+                // In that case we change t to a LowerCaseClass to let the
+                // compiler skip the resolving at several places in this class.
+                if (Character.isLowerCase(name.charAt(0))) {
+                  t = new LowerCaseClass(name);
+                }
+                isClass = resolve(t);
+            }
+            if (isClass) {
                 // the name is a type so remove it from the scoping
                 // as it is only a classvariable, it is only in
                 // referencedClassVariables, but must be removed
@@ -596,6 +832,11 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         return ve;
     }
 
+    private boolean testVanillaNameForClass(String name) {
+        if (name==null || name.length()==0) return false;
+        return !Character.isLowerCase(name.charAt(0));
+    }
+
     protected Expression transformBinaryExpression(BinaryExpression be) {
         Expression left = transform(be.getLeftExpression());
         int type = be.getOperation().getType();
@@ -665,18 +906,20 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
         Expression left = transform(oldLeft);
         if (left != oldLeft) {
             ClassExpression ce = (ClassExpression) left;
-            addError("you tried to assign a value to " + ce.getType().getName(), oldLeft);
+            addError("you tried to assign a value to the class " + ce.getType().getName(), oldLeft);
             return de;
         }
         Expression right = transform(de.getRightExpression());
         if (right == de.getRightExpression()) return de;
-        return new DeclarationExpression((VariableExpression) left, de.getOperation(), right);
+        DeclarationExpression newDeclExpr = new DeclarationExpression((VariableExpression) left, de.getOperation(), right);
+        newDeclExpr.setSourcePosition(de);
+        return newDeclExpr;
     }
 
     protected Expression transformAnnotationConstantExpression(AnnotationConstantExpression ace) {
         AnnotationNode an = (AnnotationNode) ace.getValue();
         ClassNode type = an.getClassNode();
-        resolveOrFail(type, "unable to find class for annotation", an);
+        resolveOrFail(type, ", unable to find class for annotation", an);
         for (Iterator iter = an.getMembers().entrySet().iterator(); iter.hasNext();) {
             Map.Entry member = (Map.Entry) iter.next();
             Expression memberValue = (Expression) member.getValue();
@@ -695,11 +938,18 @@ public class ResolveVisitor extends ClassCodeExpressionTransformer {
             //skip builtin properties
             if (an.isBuiltIn()) continue;
             ClassNode type = an.getClassNode();
-            resolveOrFail(type, "unable to find class for annotation", an);
+            resolveOrFail(type, ",  unable to find class for annotation", an);
             for (Iterator iter = an.getMembers().entrySet().iterator(); iter.hasNext();) {
                 Map.Entry member = (Map.Entry) iter.next();
                 Expression memberValue = (Expression) member.getValue();
-                member.setValue(transform(memberValue));
+                Expression newValue = transform(memberValue);
+                member.setValue(newValue);
+                if (newValue instanceof PropertyExpression) {
+                    PropertyExpression pe = (PropertyExpression) newValue;
+                    if (!(pe.getObjectExpression() instanceof ClassExpression)) {
+                        addError("unable to find class for enum",pe.getObjectExpression());
+                    }
+                }
             }
         }
     }
