@@ -1,5 +1,5 @@
 /*
- * Copyright 2003-2007 the original author or authors.
+ * Copyright 2003-2008 the original author or authors.
  *
  * Licensed under the Apache License, Version 2.0 (the "License");
  * you may not use this file except in compliance with the License.
@@ -30,14 +30,14 @@ import java.util.*;
 import java.util.regex.Matcher;
 
 public class DefaultTypeTransformation {
-    
+
     protected static final Object[] EMPTY_ARGUMENTS = {};
     protected static final BigInteger ONE_NEG = new BigInteger("-1");
-    
+
     //  --------------------------------------------------------
     //                  unboxing methods
-    //  --------------------------------------------------------       
-    
+    //  --------------------------------------------------------
+
     public static byte byteUnbox(Object value) {
         Number n = castToNumber(value);
         return n.byteValue();
@@ -74,12 +74,12 @@ public class DefaultTypeTransformation {
     public static double doubleUnbox(Object value) {
         Number n = castToNumber(value);
         return n.doubleValue();
-    } 
+    }
 
     //  --------------------------------------------------------
     //                  boxing methods
-    //  --------------------------------------------------------       
-    
+    //  --------------------------------------------------------
+
     public static Object box(boolean value) {
         return value ? Boolean.TRUE : Boolean.FALSE;
     }
@@ -111,7 +111,7 @@ public class DefaultTypeTransformation {
     public static Object box(double value) {
         return new Double(value);
     }
-    
+
     public static Number castToNumber(Object object) {
         if (object instanceof Number)
             return (Number) object;
@@ -129,7 +129,7 @@ public class DefaultTypeTransformation {
         }
         throw new GroovyCastException(object,Number.class);
     }
-    
+
     public static boolean castToBoolean(Object object) {
     	if (object == null) {
     		return false;
@@ -162,11 +162,11 @@ public class DefaultTypeTransformation {
         if (object instanceof CharSequence) {
         	CharSequence string =  (CharSequence) object;
             return string.length() > 0;
-        } 
+        }
         if (object instanceof Object[]) {
         	Object[] array =  (Object[]) object;
             return array.length > 0;
-        } 
+        }
         if (object instanceof Character) {
             Character c = (Character) object;
             return c.charValue() != 0;
@@ -177,10 +177,10 @@ public class DefaultTypeTransformation {
         }
         return true;
     }
-    
+
     public static char castToChar(Object object) {
         if (object instanceof Character) {
-            return ((Character) object).charValue();            
+            return ((Character) object).charValue();
         } else if (object instanceof Number) {
             Number value = (Number) object;
             return (char) value.intValue();
@@ -194,7 +194,7 @@ public class DefaultTypeTransformation {
             }
         }
     }
-    
+
     public static Object castToType(Object object, Class type) {
         if (object == null) {
             return null;
@@ -225,7 +225,7 @@ public class DefaultTypeTransformation {
                 if (type.isAssignableFrom(ArrayList.class) && (Modifier.isAbstract(modifiers) || Modifier.isInterface(modifiers))) {
                     answer = new ArrayList();
                 } else {
-                    // lets call the collections constructor
+                    // let's call the collections constructor
                     // passing in the list wrapper
                     try {
                         answer = (Collection) type.newInstance();
@@ -289,7 +289,7 @@ public class DefaultTypeTransformation {
             }
         } else if (type.isPrimitive()) {
             if (type == boolean.class) {
-               return box(booleanUnbox(object)); 
+               return box(booleanUnbox(object));
             } else if (type == byte.class) {
                 return box(byteUnbox(object));
             } else if (type == char.class) {
@@ -315,8 +315,10 @@ public class DefaultTypeTransformation {
         }
         Object[] args = null;
         if (object instanceof Collection) {
-            Collection list = (Collection) object;
-            args = list.toArray();
+            // let's try invoke the constructor with the list as arguments
+            // such as for creating a Dimension, Point, Color etc.
+            Collection collection = (Collection) object;
+            args = collection.toArray();
         } else if (object instanceof Object[]) {
             args = (Object[]) object;
         } else if (object instanceof Map) {
@@ -325,14 +327,12 @@ public class DefaultTypeTransformation {
             args[0] = object;
         }
         if (args != null) {
-            // lets try invoke the constructor with the list as arguments
-            // such as for creating a Dimension, Point, Color etc.
             try {
                 return InvokerHelper.invokeConstructorOf(type, args);
             } catch (InvokerInvocationException iie){
                 throw iie;
             } catch (Exception e) {
-                // lets ignore exception and return the original object
+                // let's ignore exception and return the original object
                 // as the caller has more context to be able to throw a more
                 // meaningful exception
             }
@@ -415,7 +415,7 @@ public class DefaultTypeTransformation {
         }
         return array;
     }
-    
+
     public static Collection asCollection(Object value) {
         if (value == null) {
             return Collections.EMPTY_LIST;
@@ -481,7 +481,7 @@ public class DefaultTypeTransformation {
 
         return false;
     }
-    
+
     /**
      * Allows conversion of arrays into a mutable List
      *
@@ -499,7 +499,7 @@ public class DefaultTypeTransformation {
         }
         return list;
     }
-    
+
     public static Object[] primitiveArrayBox(Object array) {
         int size = Array.getLength(array);
         Object[] ret = (Object[]) Array.newInstance(ReflectionCache.autoboxType(array.getClass().getComponentType()), size);
@@ -508,11 +508,15 @@ public class DefaultTypeTransformation {
         }
         return ret;
     }
-    
+
     /**
      * Compares the two objects handling nulls gracefully and performing numeric type coercion if required
      */
     public static int compareTo(Object left, Object right) {
+        return compareToWithEqualityCheck(left, right, false);
+    }
+
+    private static int compareToWithEqualityCheck(Object left, Object right, boolean equalityCheckOnly) {
         if (left == right) {
             return 0;
         }
@@ -526,7 +530,8 @@ public class DefaultTypeTransformation {
             if (left instanceof Number) {
                 if (isValidCharacterString(right)) {
                     return castToChar(left) - castToChar(right);
-                } else if (right instanceof Character || right instanceof Number) {
+                }
+                if (right instanceof Character || right instanceof Number) {
                     return DefaultGroovyMethods.compareTo((Number) left, castToNumber(right));
                 }
             }
@@ -534,14 +539,14 @@ public class DefaultTypeTransformation {
                 if (isValidCharacterString(right)) {
                     return castToChar(left) - castToChar(right);
                 }
-                else if (right instanceof Number) {
+                if (right instanceof Number) {
                     return castToChar(left) - castToChar(right);
                 }
             }
             else if (right instanceof Number) {
                 if (isValidCharacterString(left)) {
                     return castToChar(left) - castToChar(right);
-                } 
+                }
             }
             else if (left instanceof String && right instanceof Character) {
                 return ((String) left).compareTo(right.toString());
@@ -549,19 +554,26 @@ public class DefaultTypeTransformation {
             else if (left instanceof String && right instanceof GString) {
                 return ((String) left).compareTo(right.toString());
             }
-            Comparable comparable = (Comparable) left;
-            return comparable.compareTo(right);
+            if (!equalityCheckOnly || left.getClass().isAssignableFrom(right.getClass())
+                    || right.getClass().isAssignableFrom(left.getClass())
+                    || (left instanceof GString && right instanceof String)) {
+                Comparable comparable = (Comparable) left;
+                return comparable.compareTo(right);
+            }
         }
 
+        if (equalityCheckOnly) {
+            return -1; // anything other than 0
+        }
         throw new GroovyRuntimeException("Cannot compare " + left.getClass().getName() + " with value '" +
                 left + "' and " + right.getClass().getName() + " with value '" + right + "'");
     }
-    
+
     public static boolean compareEqual(Object left, Object right) {
         if (left == right) return true;
         if (left == null || right == null) return false;
         if (left instanceof Comparable) {
-            return compareTo(left, right) == 0;
+            return compareToWithEqualityCheck(left, right, true) == 0;
         }
         // handle arrays on both sides as special case for efficiency
         Class leftClass = left.getClass();
