@@ -73,6 +73,7 @@ public class JavaStubGenerator
             public void addCovariantMethods(ClassNode cn) {}
             protected void addTimeStamp(ClassNode node) {}
             protected void addInitialization(ClassNode node) {}
+            public void visitProperty(PropertyNode node) {}            
         };
         verifier.visitClass(classNode);
         
@@ -417,6 +418,17 @@ public class JavaStubGenerator
 
         printParams(methodNode, out);
 
+        ClassNode[] exceptions = methodNode.getExceptions();
+        for (int i=0; i<exceptions.length; i++) {
+            ClassNode exception = exceptions[i];
+            if (i==0) {
+                out.print("throws ");
+            } else {
+                out.print(", ");
+            }
+            printType(exception,out);
+        }
+
         if ((methodNode.getModifiers() & Opcodes.ACC_ABSTRACT) != 0) {
             out.println(";");
         } else {
@@ -511,19 +523,20 @@ public class JavaStubGenerator
         if (genericsType.isPlaceholder()) {
             out.print(genericsType.getName());
         } else {
-            printTypeName(genericsType.getType(), out);
-            ClassNode[] upperBounds = genericsType.getUpperBounds();
-            ClassNode lowerBound = genericsType.getLowerBound();
-            if (upperBounds!=null) {
-                out.print(" extends ");
-                for (int i = 0; i < upperBounds.length; i++) {
-                    printType(upperBounds[i],out);
-                    if (i+1<upperBounds.length) out.print(" & ");
-                }
-            } else if (lowerBound!=null) {
-                out.print(" super ");
-                printType(lowerBound,out);
-            }       
+            printType(genericsType.getType(),out);
+        }
+
+        ClassNode[] upperBounds = genericsType.getUpperBounds();
+        ClassNode lowerBound = genericsType.getLowerBound();
+        if (upperBounds != null) {
+            out.print(" extends ");
+            for (int i = 0; i < upperBounds.length; i++) {
+                printType(upperBounds[i], out);
+                if (i + 1 < upperBounds.length) out.print(" & ");
+            }
+        } else if (lowerBound != null) {
+            out.print(" super ");
+            printType(lowerBound, out);
         }
     }
 
